;**** A P P L I C A T I O N   N O T E   ************************************
;*
;* PHOENIXstudios 230V 128 Kanal PC_DIMMER Firmware
;* Copyright (c) 2006-2019 by PHOENIXstudios
;*
;* Chip-Typ: Atmel AVR AT90S2313/ATtiny2313
;* Chip-Typ: Atmel AVR AT90S2313/ATtiny2313
;*
;* Firmware-Revision: 2.4 (with Diagnostic and SoftID)
;* Neue LED-Zuordnung fr DMX512-Platine: 
;* Last updated: 26.02.2019
;*
;*
;* Updates: http://www.pcdimmer.de
;*
;***************************************************************************
; This program is free software; you can redistribute it and/or 
; modify it under the terms of the GNU General Public License 
; as published by the Free Software Foundation; either version2 of 
; the License, or (at your option) any later version. 
;
; This program is distributed in the hope that it will be useful, 
; but WITHOUT ANY WARRANTY; without even the implied warranty of 
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
; General Public License for more details. 
;
; If you have no copy of the GNU General Public License, write to the 
; Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
;***************************************************************************

.include "2313def.inc"
.DEVICE AT90S2313

;********* SETUP THIS FIRMWARE VIA COMPILER-DEFINES ************************
;Select crystal-frequency
	#define CONFIG_CRYSTAL_7372800HZ
	;#define CONFIG_CRYSTAL_8000000HZ

;Select uart-baudrate
	#define CONFIG_BAUD_115200
	;#define CONFIG_BAUD_57600
	;#define CONFIG_BAUD_38400
	;#define CONFIG_BAUD_9600

;Select desired main-option
	#define CONFIG_MODE_TRIAC
	;#define CONFIG_MODE_DEMUX

;Select mains-frequency / Soft-PWM-frequency
	#define CONFIG_FREQ_50HZ
	;#define CONFIG_FREQ_60HZ

;Select if outputs should be inverted (nescessary for TRIAC-output, but has to be disabled for DEMUX)
	#define CONFIG_INVERT_OUTPUT

;Select desired dimmer-curve
	#define CONFIG_CURVE_Uavg
	;#define CONFIG_CURVE_Linear
	;#define CONFIG_CURVE_Switch
	;#define CONFIG_CURVE_Power
	;#define CONFIG_CURVE_logbeta5
	;#define CONFIG_CURVE_logbeta25

;Select the desired address-type
	#define CONFIG_ADDRESS_Hardcoded
	;#define CONFIG_ADDRESS_DIP
	;#define CONFIG_ADDRESS_SoftID

;Address of this chip (if set by software without DIP-switches)
	.EQU CHIPADDRESS = 0b00000000	;Feste Adresse 0b0xxxx000
	;			 0b0xxxx000			    0b0xxxx000				0b0xxxx000				0b0xxxx000
	; Devie  1 = 0b00000000	Device  2 = 0b00001000	Device  3 = 0b00010000	Device  4 = 0b00011000
	; Devie  5 = 0b00100000	Device  6 = 0b00101000	Device  7 = 0b00110000	Device  8 = 0b00111000
	; Devie  9 = 0b01000000	Device 10 = 0b01001000	Device 11 = 0b01010000	Device 12 = 0b01011000
	; Devie 13 = 0b01100000	Device 14 = 0b01101000	Device 15 = 0b01110000	Device 16 = 0b01111000

;Choose if you want to use PD5 for activity LED or to use it as an additional output that can be controlled via RS232
	#define CONFIG_PD5_LED	; blinking LED if a valid RS232-frame is recognized
	;#define CONFIG_PD5_AUX	; additional output that can be controlled via RS232

;******* END OF SETUP ********************************************************


; Information about the expected RS232-frame:
	;6 bytes in a frame - bit 7 is used as frame start and only first byte has 7th bit set to high
	;BYTE0: b7=1	b6-b3:device-address	b2-b0:channel
	;BYTE1: b7=0	b6=PD6	b5=PD5	b4=set value as new SoftID to eeprom	b3=request channel-values over UART	b2=request device-information over UART b1-b0=step
	;BYTE2: b7=0	b6:P b5-b0:FADE19-FADE13	
	;BYTE3: b7=0	b6-b0:FADE12,FADE6
	;BYTE4: b7=0	b6-b0:FADE5 ,FADE0
	;BYTE5: b7=0 	b6-b0:END

	; EXAMPLE
	;BYTE0: 1 A A A A C C C		10001010	Channel 3 of device 2
	;BYTE1: 0 B B A V I S S		00000000	no step-size
	;BYTE2: 0 F F F F F F F		00000000	no fade-time
	;BYTE3: 0 F F F F F F F		00000000	no fade-time
	;BYTE4: 0 F F F F F F F		00000000	no fade-time
	;BYTE5: 0 E E E E E E E		00111111	50% brightness (63) - inverted data!




#ifdef CONFIG_FREQ_50HZ
	#ifdef CONFIG_CRYSTAL_7372800HZ
		.EQU CLOCK = 7372800
		.EQU PFM_TIME_H=0x01	;PFM_TIME_HL=(7,3728*10^6 Hz/(250*2*50Hz))-1= ~294 = 0x0126
		.EQU PFM_TIME_L=0x26
	#endif
	#ifdef CONFIG_CRYSTAL_8000000HZ
		.EQU CLOCK = 8000000
		.EQU PFM_TIME_H=0x01	;PFM_TIME_HL=(8*10^6 Hz/(250*2*50Hz))-1= 319 = 0x013F
		.EQU PFM_TIME_L=0x3f
	#endif
#endif
#ifdef CONFIG_FREQ_60HZ
	#ifdef CONFIG_CRYSTAL_7372800HZ
		.EQU CLOCK = 7372800
		.EQU PFM_TIME_H=0x00	;PFM_TIME_HL=(7,3728*10^6 Hz/(250*2*60Hz))-1= ~245 = 0x00F5
		.EQU PFM_TIME_L=0xF5
	#endif
	#ifdef CONFIG_CRYSTAL_8000000HZ
		.EQU CLOCK = 8000000
		.EQU PFM_TIME_H=0x01	;PFM_TIME_HL=(8*10^6 Hz/(250*2*60Hz))-1= 266 = 0x010A
		.EQU PFM_TIME_L=0x0A
	#endif
#endif

#ifdef CONFIG_BAUD_115200
	#ifdef CONFIG_CRYSTAL_7372800HZ
		.EQU BAUDRATE = 115200
	#else
		#error A baudrate of 115.2kBaud is only possible with a 7.3728MHz crystal
	#endif
#endif
#ifdef CONFIG_BAUD_57600
	#ifdef CONFIG_CRYSTAL_7372800HZ
		.EQU BAUDRATE = 57600
	#else
		#error A baudrate of 57.6kBaud is only possible with a 7.3728MHz crystal
	#endif
#endif
#ifdef CONFIG_BAUD_38400
	.EQU BAUDRATE = 38400
#endif
#ifdef CONFIG_BAUD_9600
	.EQU BAUDRATE = 9600
#endif



.EQU zc_error=15	    ;zero cross error window +-(15/250)*10ms=+-0,6ms (Mains AC Peak every 5ms)

;RAM location:
;--------------------
.EQU loc_fade    =0x60	;fade register (fade_2_[x],fade_1_[x],fade_0_[x] x=channel 0..7)
.EQU loc_step    =0x78	;fade_step (step[x])
.EQU loc_end     =0x80	;end value of fading (end[x])
.EQU loc_phm     =0x88	;phase modulator registers
.EQU loc_cnt     =0x90	;fade counter  (cnt_2_[x],cnt_1_[x],cnt_0_[x])
.EQU loc_rs232   =0xA8	;RS232 buffer (6 bytes)
.EQU loc_stack   =0xDF

;Registerdefinitionen
;--------------------
.DEF TABLE=R0		;register used for flash rom dimm table
.DEF PHM0=R1		;PHase Modulator regiser 0-7
.DEF PHM1=R2
.DEF PHM2=R3
.DEF PHM3=R4
.DEF PHM4=R5
.DEF PHM5=R6
.DEF PHM6=R7
.DEF PHM7=R8
.DEF TEMP_REGS=R9	;used to safe the status register in interrupt routine sync
.DEF TEMP_REGP=R9	;used to safe the status register in interrupt routine PHM
.DEF RS232=R10		
.DEF RS232_TEMP=R11
.DEF RS232_CNT=R17	;RS232 byte counter
.DEF STEP=R18		;PHase Modulator step size
.DEF ADDRESS=R15
.DEF PHM_COUNTER=R16;PHase Modulator counter
.DEF PHM_HALF=R12	;1e,2e half mains period counter 
.DEF PHM_OUT=R13	;PHase Modulator output register
.DEF PHM_REG=R14	;PHase Modulator actual register
.DEF TEMP=R19
.DEF TEMP2=R20
.DEF RS232_OUT_BUFFER=R20
.DEF CHANNEL=R21	;actual channel
.DEF END=R22		;actual channel end value of FADE
.DEF PHM=R23		;temp register for PHM update
.DEF FADE_0=R24		;
.DEF FADE_1=R25		;|3 byte fade time value
.DEF FADE_2=R26		;
.DEF CNT_0=R27		;
.DEF CNT_1=R28		;|3 byte fade time counter	
.DEF CNT_2=R29		;
.DEF Z_L=R30		;address pointer 
.DEF Z_H=R31		;(shared with ERROR)

;EEPROM location
;--------------------
;0x00-0x01		-> Adresse
.ESEG
.db 0,0		;Standardadresse, wenn neue Firmware geladen wurde (Standard: Gerte-ID 1); Device-ID will be loaded from EEPROM 0x01 (second byte!)


.CSEG
	rjmp RESET		;reset vector
	rjmp SYNC		;INT0	      start zero cross sync
	reti			;INT1
	reti			;TIMER1 cap
	rjmp PHMB		;TIMER1 comp  START PHASE MODULATOR on timer 1 compare
	reti			;TIMER1 ovf1
	rjmp LEOF		;TIMER0 ovf0  put led off after 255ms
	reti			;UART Rx complete
	reti			;UART data register empty
	reti			;UART TX complete
	reti			;analog comparator
.ORG 0x0C


;***************************************************************************
; Betriebssystemeigenschaften ;-)
;----------------------------

	NAME:
		.db "n:PC_DIMMER",0
	VERSION:
		.db "v:2.4",0
	DATUM:
		.db "d:26.02.2019",0
	TYP:
		#ifdef CONFIG_MODE_TRIAC
			.db "t:230V~ TRIAC",0
		#endif
		#ifdef CONFIG_MODE_DEMUX
			.db "t:0-10V= DEMUX",0
		#endif
	CHIP:
		.db "c:Atmel ATtiny2313",0
;		.db "c:Atmel AT90S2313",0
	QUARZ:
		#ifdef CONFIG_CRYSTAL_7372800HZ
			.db "q:7,3728",0 				;[MHz]
		#endif
		#ifdef CONFIG_CRYSTAL_8000000HZ
			.db "q:8,0",0					;[MHz]
		#endif
	DATENRATE:
		#ifdef CONFIG_BAUD_115200
			.db "r:115,2",0					;[kBit/s]
		#endif
		#ifdef CONFIG_BAUD_57600
			.db "r:57,6",0					;[kBit/s]
		#endif
		#ifdef CONFIG_BAUD_38400
			.db "r:38,4",0					;[kBit/s]
		#endif
		#ifdef CONFIG_BAUD_9600
			.db "r:9,6",0					;[kBit/s]
		#endif
	FREQUENZ:
		#ifdef CONFIG_FREQ_50HZ
			.db "f:50",0					;[Hz]
		#endif
		#ifdef CONFIG_FREQ_60HZ
			.db "f:60",0					;[Hz]
		#endif
	DIMMKURVE:
		#ifdef CONFIG_CURVE_Uavg
			.db "k:Kurve 1: U_avg",0
		#endif
		#ifdef CONFIG_CURVE_Linear
			.db "k:Kurve 2: Linear",0
		#endif
		#ifdef CONFIG_CURVE_Switch
			.db "k:Kurve 3: Switch",0
		#endif
		#ifdef CONFIG_CURVE_Power
			.db "k:Kurve 4: Power Beta=5",0
		#endif
		#ifdef CONFIG_CURVE_logbeta5
			.db "k:Kurve 5: Power Beta=15",0
		#endif
		#ifdef CONFIG_CURVE_logbeta25
			.db "k:Kurve 6: Power Beta=25",0
		#endif

;***************************************************************************


;power compensation table

#ifdef CONFIG_CURVE_Uavg
	DIM1:
		;Uavg (Standard)
		;================
		.DB   4, 11, 15, 17, 20, 22, 24, 27, 29, 30, 32, 33, 35, 36, 38, 39
		.DB  40, 42, 43, 44, 45, 46, 47, 48, 50, 51, 52, 53, 54, 55, 56, 57
		.DB  58, 59, 59, 60, 61, 62, 63, 64, 65, 66, 66, 67, 68, 69, 70, 70
		.DB  71, 72, 73, 74, 74, 75, 76, 77, 77, 78, 79, 80, 80, 81, 82, 83
		.DB  84, 84, 85, 86, 86, 87, 88, 88, 89, 90, 90, 91, 92, 92, 93, 94
		.DB  94, 95, 96, 96, 97, 98, 98, 99,100,100,101,102,102,103,104,104
		.DB 105,106,106,107,108,108,109,109,110,111,111,112,113,113,114,114
		.DB 115,116,116,117,118,118,119,119,120,121,121,122,123,123,124,124
		.DB 125,126,126,127,127,128,129,129,130,131,131,132,132,133,134,134
		.DB 135,136,136,137,137,138,139,139,140,141,141,142,142,143,144,144
		.DB 145,146,146,147,148,148,149,150,150,151,152,152,153,154,154,155
		.DB 156,156,157,158,158,159,160,160,161,162,162,163,164,164,165,166
		.DB 166,167,168,169,170,170,171,172,173,173,174,175,176,176,177,178
		.DB 179,180,180,181,182,183,184,184,185,186,187,188,189,190,191,191
		.DB 192,193,194,195,196,197,198,199,200,202,203,204,205,206,207,208
		.DB 210,211,212,214,215,217,218,220,221,223,226,228,230,233,235,239
#endif
#ifdef CONFIG_CURVE_Linear
	DIM1:
		;Linear (z.B. fr 0-10V Anwendungen)
		;====================================
		.DB   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
		.DB  16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
		.DB  32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47
		.DB  48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
		.DB  64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79
		.DB  80, 81, 83, 84, 85, 86, 87, 88, 89, 80, 90, 91, 92, 93, 94, 95
		.DB  96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
		.DB 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
		.DB 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
		.DB 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
		.DB 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
		.DB 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
		.DB 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
		.DB 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
		.DB 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
		.DB 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255
#endif
#ifdef CONFIG_CURVE_Switch
	DIM1:
		;Switch (z.B. fr Switchpacks)
		;==============================
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
		.DB 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
#endif
#ifdef CONFIG_CURVE_Power
	DIM1:
		;Power
		;======
		.DB  11, 22, 27, 31, 34, 37, 39, 41, 43, 45, 46, 48, 50, 51, 52, 54
		.DB  55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70
		.DB  70, 71, 72, 73, 74, 74, 75, 76, 76, 77, 78, 79, 79, 80, 81, 81
		.DB  82, 83, 83, 84, 85, 85, 86, 86, 87, 88, 88, 89, 90, 90, 91, 91
		.DB  92, 92, 93, 94, 94, 95, 95, 96, 97, 97, 98, 98, 99, 99,100,100
		.DB 101,101,102,102,103,104,104,105,105,106,106,107,107,108,108,109
		.DB 109,110,110,111,111,112,112,113,113,114,114,115,115,116,116,117
		.DB 117,118,118,119,119,120,120,121,121,122,122,123,123,124,124,125
		.DB 125,125,126,126,127,127,128,128,129,129,130,130,131,131,132,132
		.DB 133,133,134,134,135,135,136,136,137,137,138,138,139,139,140,140
		.DB 141,141,142,142,143,143,144,144,145,145,146,146,147,148,148,149
		.DB 149,150,150,151,151,152,152,153,153,154,155,155,156,156,157,158
		.DB 158,159,159,160,160,161,162,162,163,164,164,165,165,166,167,167
		.DB 168,169,169,170,171,171,172,173,174,174,175,176,176,177,178,179
		.DB 180,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194
		.DB 195,196,198,199,200,202,204,205,207,209,211,213,216,219,223,236
#endif
#ifdef CONFIG_CURVE_logbeta5
	DIM1:
		;log Power BETA=5
		;=================
		.DB  13, 25, 31, 36, 39, 42, 45, 47, 49, 51, 53, 55, 57, 59, 60, 61
		.DB  63, 64, 65, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79
		.DB  80, 81, 82, 83, 84, 85, 85, 86, 87, 88, 88, 89, 90, 91, 91, 92
		.DB  93, 94, 94, 95, 96, 96, 97, 98, 98, 99,100,100,101,102,102,103
		.DB 103,104,105,105,106,107,107,108,108,109,109,110,111,111,112,112
		.DB 113,113,114,114,115,115,116,116,117,118,118,119,119,120,120,121
		.DB 121,122,122,123,123,124,124,125,125,126,126,127,127,128,128,129
		.DB 129,130,130,131,131,132,132,133,133,133,134,134,135,135,136,136
		.DB 137,137,138,138,139,139,140,140,141,141,141,142,142,143,143,144
		.DB 144,145,145,146,146,147,147,148,148,149,149,149,150,150,151,151
		.DB 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
		.DB 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
		.DB 168,168,169,170,170,171,171,172,172,173,173,174,175,175,176,176
		.DB 177,177,178,179,179,180,180,181,182,182,183,184,184,185,186,186
		.DB 187,188,189,189,190,191,192,193,193,194,195,196,197,198,199,200
		.DB 201,202,203,204,206,207,208,210,211,213,215,217,219,222,225,236
#endif
#ifdef CONFIG_CURVE_logbeta25
	DIM1:
		;log Power BETA=25
		;==================
		.DB  15, 30, 36, 42, 46, 49, 52, 55, 58, 60, 62, 65, 66, 68, 70, 71
		.DB  73, 74, 76, 77, 78, 80, 81, 82, 83, 85, 86, 87, 88, 89, 90, 91
		.DB  92, 93, 94, 95, 96, 96, 97, 98, 99,100,101,101,102,103,104,104
		.DB 105,106,107,107,108,109,110,110,111,112,112,113,113,114,115,115
		.DB 116,116,117,118,118,119,119,120,121,121,122,122,123,123,124,124
		.DB 125,126,126,127,127,128,128,129,129,130,130,131,131,132,132,133
		.DB 133,134,134,135,135,136,136,136,137,137,138,138,139,139,140,140
		.DB 141,141,142,142,143,143,143,144,144,145,145,146,146,147,147,147
		.DB 148,148,149,149,150,150,151,151,151,152,152,153,153,154,154,154
		.DB 155,155,156,156,157,157,157,158,158,159,159,160,160,160,161,161
		.DB 162,162,163,163,164,164,164,165,165,166,166,167,167,167,168,168
		.DB 169,169,170,170,171,171,172,172,172,173,173,174,174,175,175,176
		.DB 176,177,177,178,178,179,179,180,180,181,181,182,182,183,183,184
		.DB 184,185,185,186,186,187,188,188,189,189,190,191,191,192,192,193
		.DB 194,194,195,196,196,197,198,199,199,200,201,202,202,203,204,205
		.DB 206,207,208,209,210,211,213,214,216,217,218,220,222,225,228,236
#endif

RESET:	
	#ifdef CONFIG_ADDRESS_DIP
		#ifdef CONFIG_PD5_LED
			#error PD5 cannot be used for LED if it is used for DIP-ID
		#endif
		#ifdef CONFIG_PD5_AUX
			#error PD5 cannot be used for AUX-Output if it is used for DIP-ID
		#endif

		; PD3...PD6 are inputs for DIP-switches
		ldi temp,0b00000010	;PD1 as Output, PD3-6 as Input
		out DDRD,temp		;set data direction of port D
		ldi temp,0b11111111	
		out PORTD,temp		;set pull up to inputs PORT D
	#else
		ldi temp,0b00110010	;PD5, PD4, PD1 as Output, PD3 as Input
		out DDRD,temp		;set data direction of port D
		ldi temp,0b11001111	
		out PORTD,temp		;set pull up to inputs PORT D
	#endif

	
	ldi temp,0b11111111	
	out PORTB,temp		;set all outputs of port B to"1"
	out DDRB,temp		;set data direction of port B

	#ifndef CONFIG_ADDRESS_DIP
		cbi PORTD,DDD4		;put led off (Green LED on new DMX512-PCBs)
	#endif
	#ifdef CONFIG_PD5_LED
		sbi PORTD,DDD5		;put led on (Yellow LED on new DMX512-PCBs)
	#endif

	ldi temp, 0x0E		;activate watchdog on 1024ms
	out WDTCR,temp
	
	ldi temp,loc_stack
	out SPL,temp		;locate stack

	clr Z_H
	ldi Z_L,1			;z:=address(PHM0)
	ldi temp,0xFF
  SPHM:
  	st z+,temp			;set PHM0..PHM7 to 0xFF
	cpi Z_L,9
	brlo SPHM
 	ldi Z_L,loc_fade
  SETR:
  	st z+,temp
	cpi Z_L,loc_cnt		;fill ram with 0xFF
	brlo SETR		
	
	ldi temp,CLOCK/(BAUDRATE*16)-1   ;calculate correct clock-divider
	out UBRR,temp		;set UART to Baudrate set above
	sbi UCR,RXCIE		;receive complete interrupt enabled
	sbi UCR, RXEN		;receive enabled
	sbi UCR, TXEN      	;rs232out enabled

	#ifdef CONFIG_ADDRESS_Hardcoded
		ldi TEMP, CHIPADDRESS	;Feste Adresse durch Laden einer Konstanten (Software)
		mov ADDRESS, TEMP
	#endif
	#ifdef CONFIG_ADDRESS_DIP
		in TEMP,PIND			;Variable Adresse durch Laden von I/O Register (DIP-Schalter) in Universalregister
		andi TEMP,0b01111000
		mov ADDRESS, TEMP
	#endif
	#ifdef CONFIG_ADDRESS_SoftID
		rcall ADDRESS_read		;nur wenn Adresse aus EEPROM (SoftID)
	#endif

	; Statusmeldung ber RS232
	rcall ABOUT
	rcall ZC_SYNC
	cbi UCR, TXEN		;rs232out disabled

	ldi temp,0b00001001
	out TCCR1B,temp		;set timer1_clk=clk=1 and clr on compare match

	ldi temp,PFM_TIME_H
	out OCR1AH,temp		;set timer1 H & L compare register
	ldi temp,PFM_TIME_L
	out OCR1AL,temp	

	clr RS232_CNT		;reset RS232 input word counter
	
	ldi temp,0b00000010	;set IRQ INT0 to falling edge
	out MCUCR,temp	

	ldi temp,0b01000000
	out GIMSK,temp		;enable INT0

	clr PHM_HALF
	#ifdef CONFIG_MODE_TRIAC
		ldi PHM_COUNTER,14	;set PHM_COUNT to start up value for SYNC routine (230V~ TRIAC)
	#endif
	#ifdef CONFIG_MODE_DEMUX
		ldi PHM_COUNTER,0	;set PHM_COUNT to start up value for SYNC routine (0-10V= DEMUX)
	#endif
	sei					;enable interrupts

  TPHC:
	wdr
	cpi PHM_COUNTER,0	;wait until PHM_COUNTER==0 (set by INT0 sync routine with ZC-Signal)
	brne TPHC
	ldi temp,1<<OCIE1A	;enable timer1 interrupt
	ori temp,1<<TOIE0	;enable timer0 overflow
	out TIMSK,temp

	; Statusmeldung ber RS232
	ldi temp,CLOCK/(BAUDRATE*16)-1   ;calculate correct clock-divider
	out UBRR,temp		;set UART to Baudrate set above
	sbi UCR,RXCIE		;receive complete interrupt enabled
	sbi UCR, RXEN		;receive enabled
	sbi UCR, TXEN		;rs232out enabled
	rcall ZC_OK
	cbi UCR, TXEN		;rs232out disabled

	#ifndef CONFIG_ADDRESS_DIP
		sbi PORTD,DDD4		;put led on (Green LED on new DMX512-PCBs)
	#endif
	#ifdef CONFIG_PD5_LED
		sbi PORTD,DDD5		;put led on (Yellow LED on new DMX512-PCBs)
	#endif

main:
	sleep
   	rjmp main;	

;***************************************************************************
; Start of ZeroCrossing-Detection
;***************************************************************************

SYNC:
	;PHM_COUNTER range 0..249 (dividefactor=250)(TIMER=8*10^6 Hz/(250*2*50Hz) =320	or  (TIMER=7,3728*10^6 Hz/(250*2*50Hz) =295
	;sync Timer to mains
	in TEMP_REGS,SREG	;store SREG

	tst PHM_HALF
	brne NOSY			;if first half no sync allowed

	cpi PHM_COUNTER,0	
	breq NOSY			;IF PHM_COUNTER==0 then COUNTER is in sync
				
	cpi PHM_COUNTER,249-zc_error
	brsh INCP			;if 249-zc_error<PHM_COUNTER<250 then PHM_COUNTER:=PHM_COUNTER+1

	cpi PHM_COUNTER,zc_error+1
	brsh NOSY			;if zc_error<PHM_COUNT<249-zc_error then NOSYnc (window)

	dec PHM_COUNTER		;if 0<PHM_COUNTER<zc_error then PHM_COUNTER:=PHM_COUNTER-1
	rjmp NOSY
 INCP:
 	inc PHM_COUNTER
	cpi PHM_COUNTER,250
	brne NOSY
	ldi PHM_COUNTER,0	;if PHM_COUNTER=250 then PHM_COUNTER:=0	
 NOSY:
 	out SREG,TEMP_REGS	;recall SREG 
 	reti

;***************************************************************************
; End of ZeroCrossing-Detection
;***************************************************************************


PHMB:
	;8-channel PHM(PHase-Modulator) 
	;30,5 cycles	
	in TEMP_REGP,SREG	;store SREG
	cp PHM0,PHM_COUNTER	;if PHM0<PHM_COUNTER carry:=1
	ror PHM_REG		;shift CARRY right in PHM_REG
	cp PHM1,PHM_COUNTER
	ror PHM_REG	
	cp PHM2,PHM_COUNTER
	ror PHM_REG
	cp PHM3,PHM_COUNTER
	ror PHM_REG
	cp PHM4,PHM_COUNTER
	ror PHM_REG
	cp PHM5,PHM_COUNTER
	ror PHM_REG
	cp PHM6,PHM_COUNTER
	ror PHM_REG
	cp PHM7,PHM_COUNTER
	ror PHM_REG
	mov PHM_OUT,PHM_REG
	cpi PHM_COUNTER,15	;if PHM_COUNTER<15 OR>220 then PHM_OUT:=0 safety margin
	brlo CLRP		;safety margin is used to avoid triggering in wrong
	cpi PHM_COUNTER,220	;period half.(inductive loads)(DON'T CHANGE THESE LIMITS!!!)
	brlo COPH
  CLRP:	clr PHM_OUT
  COPH:
	#ifdef CONFIG_INVERT_OUTPUT
  		com PHM_OUT		;invert PHM_OUT
	#endif
	out PORTB,PHM_OUT	;set PHM output

	cpi PHM_COUNTER,220
	brne CNTM
	rcall UPDP			;store new values to PHM-regs
	wdr 
	com PHM_HALF		;set to next half period
  CNTM:	inc PHM_COUNTER
	cpi PHM_COUNTER,250
	brne COCH
	clr PHM_COUNTER		;if PHM_COUNTER=250 then PHM_COUNTER:=0
  COCH:	cpi PHM_COUNTER,100	;if 100<=PHM_COUNTER<108 update channel
	brlt NXTC 
	cpi PHM_COUNTER,108
	brsh NXTC
	mov CHANNEL,PHM_COUNTER
	subi CHANNEL,100	;CHANNEL:=PHM_COUNTER-100
	rcall UPDC		;call UPDate Channel 
  NXTC:
  	rcall RS232_IN		;call get data from RS232

 R_PHM:	
 	out SREG,TEMP_REGP	;recall SREG
	reti



UPDC:	;UPDate Channel[CHANNEL]

	clr Z_H
	ldi Z_L,loc_fade
	add Z_L,CHANNEL
	ld FADE_0,z
	ldd FADE_1,z+8
	ldd FADE_2,z+16 	;FADE:=FADE[channel]
	ldi Z_L,loc_end
	add Z_L,CHANNEL
	ld END,z		;END:=END[channel]
	clr TEMP
	cp  FADE_0,TEMP
	cpc FADE_1,TEMP
	cpc FADE_2,TEMP
	brne PHME
	ldi Z_L,loc_phm
	add Z_L,CHANNEL
	st z,END		;IF FADE[CHANNEL]=0 THEN PHM[channel]:=END
	rjmp UPDR
PHME:	ldi Z_L,loc_phm
	add Z_L,CHANNEL
	ld PHM,z		;PHM:=PHM[channel]
	cp END,PHM		;if PHM[channel]==END[channel] UPDR (update ready)
	breq UPDR
	ldi Z_L,loc_cnt
	add Z_L,CHANNEL
	ld CNT_0,z
	ldd CNT_1,z+8
	ldd CNT_2,z+16		;CNT:=CNT[channel]

	clr TEMP
	cpi CNT_0,2
	cpc CNT_1,TEMP
	cpc CNT_2,TEMP
	brsh CNTD		;if CNT>=2 continue count down 
		
	ldi Z_L,loc_cnt
	add Z_L,CHANNEL
	st z,FADE_0
	std z+8,FADE_1
	std z+16,FADE_2		;CNT[channel]:=FADE[channel]  (refresh count registers)  
	
		
	ldi Z_L,loc_step
	add Z_L,CHANNEL
	ld STEP,z		;STEP:=STEP[channel]   
	cp PHM,END
	brsh PHGR		
	add PHM,STEP		;if PHM<END then PHM:=PHM+STEP				 			
	brcs MEND		;if PHM>255(carry=1) OR PHM>=END then PHM:=END
	cp PHM,END
	brsh MEND		
	rjmp STPH
  PHGR:	sub PHM,STEP		;if PHM>END then PHM:=PHM-STEP
	brcs MEND		;if PHM<0(carry=1) OR PHM<END  then PHM:=END
	cp PHM,END
	brsh STPH
  MEND:	mov PHM,END		  
  STPH:	ldi Z_L,loc_phm
	add Z_L,CHANNEL
	st z,PHM		;PHM[channel]:=PHM
	rjmp UPDR

 CNTD:  subi CNT_0,1
	sbci CNT_1,0
	sbci CNT_2,0		;CNT:=CNT-1

	ldi Z_L, loc_cnt
	add Z_L,CHANNEL
	st z,CNT_0
	std z+8,CNT_1
	std z+16,CNT_2		;CNT[channel]:=CNT
	 
  UPDR:	ret

UPDP:	;update PHM-registers:
	
	clr Z_H
	clr CHANNEL
  NCTC:	ldi Z_L,loc_phm
	add Z_L,CHANNEL
	ld PHM,z		;PHM:=PHM[channel]
	ldi Z_L,low(2*DIM1)	;z:=address table 2*DIM1 Z_L b0=0 low byte b0=1 high byte				;
	ldi Z_H,high(2*DIM1)	;pointer: Z_H b0 and Z_L b7-b1
	clr TEMP

  ADPH:	add Z_L,PHM
	adc Z_H,TEMP	
	lpm			;TABLE:=DIM1[PHM]
	clr Z_H
	mov Z_L,CHANNEL
	std z+1,TABLE		;PHM_reg[channel]:=TABLE
	inc CHANNEL
	cpi channel,8		;repeat until channel=8
	BRNE NCTC
	ret


RS232_IN:
 
	;6 bytes in a frame - b7 is used as frame start
	;B0: b0,7=1	b0,6-b0,3:pcb address	b0,2-b0,0:channel
	;B1: b1,7-b1,2=0 b1,1-b1,0=step
	;B2: b2,7=0	b2,6:P b2,5-b2,0:FADE19-FADE13	
	;B3: b3,7=0	b3,6-b3,0:FADE12,FADE6
	;B4: b4,7=0	b4,6-b4,0:FADE5 ,FADE0
	;B5: b5,7=0 	b5,6-b4,0:END

	;B0: 1 A A A A C C C
	;B1: 0 0 0 0 0 0 S S
	;B2: 0 F F F F F F F
	;B3: 0 F F F F F F F
	;B4: 0 F F F F F F F
	;B5: 0 E E E E E E E 

 	sbis USR,RXC		;exit if no new word is received
	rjmp EXRS

  	in RS232,UDR		;read RS232
	sbic USR,FE		;exit if no stop bit in word
	rjmp EXRS

	cpi RS232_CNT,0
	brne IFC5		;if RS232_CNT<>0  IFC5 

	sbrs RS232,7
	rjmp EXRS		;if bit_7=0 exit (no start byte)

	;check address
	ldi TEMP,0b01111000		;Lade Register TEMP mit Wert
	mov RS232_TEMP,RS232	;Kopiere Registerinhalt von RS232 auf RS232_TEMP
	and RS232_TEMP,TEMP		;Logisches UND zweier Register

	mov TEMP,ADDRESS
	andi TEMP,0b01111000	;Logisches UND Register mit Wert
	eor TEMP,RS232_TEMP		;Exklusives ODER zweier Register
	brne EXRS				;Sprung wenn ungleich - if PIND(b6,b5,b4,b3)<>RS232_0(b6,b5,b4,b3) THEN wrong address EXIT

	clr Z_H				;Lsche alle Bits im Register
	ldi Z_L,loc_rs232	;Lade Register mit Wert
	st z,RS232			;RS232[0]:=RS232 
	ldi RS232_CNT,1		;Lade Register mit Wert

	rjmp EXRS

  IFC5:

	cpi RS232_CNT,6
	brsh EXRS		;if RS232_CNT>=6 EXIT
	sbrc RS232,7
	rjmp RSER		;if (RS232,7)==0 ERROR EXIT!

	clr Z_H
	ldi Z_L,loc_rs232
	add Z_L,RS232_CNT
	st z,RS232		;RS232[RS232_CNT]:=RS232

	inc RS232_CNT

	cpi RS232_CNT,6
	brne EXRS

	rcall LEON		;if one frame is received put led on for 255ms
	rcall COPY		;copy RS232 data and SHIFT(FADE[0..2])

  RSER:	clr RS232_CNT	

  EXRS: ret

COPY:
	;if one frame received data can be decoded and saved to the channel memory or eeprom
	;copy RS232 data to channels 	

	clr Z_H
	ldi Z_L,loc_rs232
	ld RS232,z		;RS232:=RS232[0]
	ldi CHANNEL,0b00000111
	and CHANNEL,RS232


	ldi Z_L,loc_rs232
	ldd RS232,z+1		;RS232:=RS232[1]
	ldi STEP,0b00000011
	and STEP,RS232		;STEP:=RS232[1] AND b00000011
	inc step

	; Statusinformationen per RS232 senden, wenn Bit2 gesetzt
	sbrs RS232,2
	rjmp NOSTATUS
		sbi UCR, TXEN		;rs232out enabled
		rcall ABOUT
		rcall ZC_OK
		cbi UCR, TXEN		;rs232out disabled
		rjmp ENDC		; nach Statusbericht sicherstellen, dass kein Kanal gendert wird
	NOSTATUS:

	; Kanalinformationen per RS232 senden, wenn Bit3 gesetzt
	sbrs RS232,3
	rjmp NOCHANNELINFO
		sbi UCR, TXEN		;rs232out enabled
		rcall KANALWERTE
		cbi UCR, TXEN		;rs232out disabled
		rjmp ENDC		; nach Statusbericht sicherstellen, dass kein Kanal gendert wird
	NOCHANNELINFO:

	; Adresse per RS232 ndern, wenn Bit4 gesetzt
	sbrs RS232,4
	rjmp NOADDRESSCHANGE
		ldi Z_L,loc_rs232
		ldd TEMP,z+5
		mov ADDRESS,TEMP
		sbi UCR, TXEN		;rs232out enabled
		rcall ABOUT
		cbi UCR, TXEN		;rs232out disabled
		rcall ADDRESS_write	;neue Adresse in EEPROM schreiben
		rjmp ENDC		; nach Statusbericht sicherstellen, dass kein Kanal gendert wird
	NOADDRESSCHANGE:

	; Routine, wenn Bit5 gesetzt
	#ifdef CONFIG_PD5_AUX
		sbrs RS232,5
		rjmp NOBIT5
			sbi  PORTD,DDD5
			rjmp ENDC
		NOBIT5:
			cbi  PORTD,DDD5
	#endif

	#ifndef CONFIG_ADDRESS_DIP
		sbrs RS232,6
		; Routine, wenn Bit6 gesetzt
		rjmp NOBIT6
			sbi  PORTD,DDD6
			rjmp ENDC
		NOBIT6:
			cbi  PORTD,DDD6
	#endif

	ldi Z_L,loc_rs232
	ldd FADE_0,z+4
	ldd FADE_1,z+3
	ldd FADE_2,z+2

	lsl FADE_0		;-0c6c5c4c3c2c1c0  -0b6b5b4b3b2b1b0  a6a5a4a3a2a1a0-0  C=0
	lsr FADE_1		;-0c6c5c4c3c2c1c0  -0-0b6b5b4b3b2b1  a6a5a4a3a2a1a0-0  C=b0  
	ror FADE_0		;-0c6c5c4c3c2c1c0  -0-0b6b5b4b3b2b1  b0a6a5a4a3a2a1a0  C=0
	lsl FADE_1		;-0c6c5c4c3c2c1c0  -0b6b5b4b3b2b1-0  b0a6a5a4a3a2a1a0  C=0
	lsl FADE_1		;-0c6c5c4c3c2c1c0  b6b5b4b3b2b1-0-0  b0a6a5a4a3a2a1a0  C=0
	lsr FADE_2		;-0-0c6c5c4c3c2c1  b6b5b4b3b2b1-0-0  b0a6a5a4a3a2a1a0  C=c0
	ror FADE_1		;-0-0c6c5c4c3c2c1  c0b6b5b4b3b2b1-0  b0a6a5a4a3a2a1a0  C=0
	lsr FADE_2		;-0-0-0c6c5c4c3c2  c0b6b5b4b3b2b1-0  b0a6a5a4a3a2a1a0  C=c1
	ror FADE_1		;-0-0-0c6c5c4c3c2  c1c0b6b5b4b3b2b1  b0a6a5a4a3a2a1a0  C=0

	ldi Z_L,loc_rs232
	ldd END,z+5
	lsl END			;END:=END*2

	ldi Z_L,loc_fade
	add Z_L,CHANNEL		
	st z,FADE_0		;FADE[CHANNEL,0]:=FADE_0
	std z+8,FADE_1		;FADE[CHANNEL,1]:=FADE_1		
	std z+16,FADE_2		;FADE[CHANNEL,2]:=FADE_2

	ldi Z_L,loc_step
	add Z_L,CHANNEL
	st z,STEP		;STEP[CHANNEL]:=STEP

	ldi Z_L,loc_end
	add Z_L,CHANNEL		;END[CHANNEL]:=RS232[5]
	st z,END	

	clr TEMP
	ldi Z_L,loc_cnt		
	add Z_L,CHANNEL
	st z,TEMP		;clear CNT[CHANNEL] to start a new proces
	std z+8,TEMP
	std z+16,TEMP	

ENDC: ret


LEOF:	;put led off
	in TEMP_REGS,SREG	;store SREG
	cbi PORTD,DDD1		;put led off (parallel to TxD)
	#ifdef CONFIG_PD5_LED
		cbi PORTD,DDD5		;put led off (Yellow LED on new DMX512-PCBs)
	#endif
	clr temp			;stop timer0
	out TCCR0,temp	
	out SREG,TEMP_REGS	;recall SREG
	reti

LEON:	;put led on and start timer0
	ldi temp,0x7F		;set timer0 to 127
	out TCNT0,temp
	ldi temp,0x05		;set prescaler timer0 to 1024
	out TCCR0,temp	
	sbi PORTD,DDD1		;put led on (parallel to TxD)
	#ifdef CONFIG_PD5_LED
		sbi PORTD,DDD5		;put led on (Yellow LED on new DMX512-PCBs)
	#endif
	ret


; TEXTAUSGABEN

ABOUT:
	; Boottext
	ldi ZL,low(NAME*2)
	ldi ZH,high(NAME*2)
	rcall RS232_OUT_STRING	

	ldi ZL,low(VERSION*2)
	ldi ZH,high(VERSION*2)
	rcall RS232_OUT_STRING	

	ldi ZL,low(DATUM*2)
	ldi ZH,high(DATUM*2)
	rcall RS232_OUT_STRING	
	
	ldi ZL,low(TYP*2)
	ldi ZH,high(TYP*2)
	rcall RS232_OUT_STRING	
	
	ldi ZL,low(CHIP*2)
	ldi ZH,high(CHIP*2)
	rcall RS232_OUT_STRING	

	ldi ZL,low(QUARZ*2)
	ldi ZH,high(QUARZ*2)
	rcall RS232_OUT_STRING	

	ldi ZL,low(DATENRATE*2)
	ldi ZH,high(DATENRATE*2)
	rcall RS232_OUT_STRING	

	ldi ZL,low(FREQUENZ*2)
	ldi ZH,high(FREQUENZ*2)
	rcall RS232_OUT_STRING	

	ldi ZL,low(DIMMKURVE*2)
	ldi ZH,high(DIMMKURVE*2)
	rcall RS232_OUT_STRING	

	;ADRESSE
	ldi RS232_OUT_BUFFER, 'a'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, ADDRESS
	andi RS232_OUT_BUFFER,0b01111000
	ror RS232_OUT_BUFFER
	ror RS232_OUT_BUFFER
	ror RS232_OUT_BUFFER
	inc RS232_OUT_BUFFER
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
ret

ZC_SYNC:
	; ZeroCrossing
	ldi RS232_OUT_BUFFER, 'z'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	#ifdef CONFIG_MODE_TRIAC
		ldi RS232_OUT_BUFFER, 'S'
		rcall RS232_OUT
		ldi RS232_OUT_BUFFER, 'Y'
		rcall RS232_OUT
	#endif
	#ifdef CONFIG_MODE_DEMUX
		ldi RS232_OUT_BUFFER, 'N'
		rcall RS232_OUT
		ldi RS232_OUT_BUFFER, 'A'
		rcall RS232_OUT
	#endif
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
ret

ZC_OK:
	; ZeroCrossing
	ldi RS232_OUT_BUFFER, 'z'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	#ifdef CONFIG_MODE_TRIAC
		ldi RS232_OUT_BUFFER, 'O'
		rcall RS232_OUT
		ldi RS232_OUT_BUFFER, 'K'
		rcall RS232_OUT
	#endif
	#ifdef CONFIG_MODE_DEMUX
		ldi RS232_OUT_BUFFER, 'N'
		rcall RS232_OUT
		ldi RS232_OUT_BUFFER, 'A'
		rcall RS232_OUT
	#endif
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
ret

KANALWERTE:
	in TEMP_REGS,SREG	;store SREG
	ldi RS232_OUT_BUFFER, '1'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM0
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '2'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM1
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '3'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM2
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '4'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM3
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '5'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM4
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '6'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM5
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '7'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM6
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, '8'
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, ':'
	rcall RS232_OUT
	mov RS232_OUT_BUFFER, PHM7
	rcall RS232_OUT
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
ret

RS232_OUT:
	sbis USR,UDRE ; test for TX register empty
	rjmp RS232_OUT ; loop until TX empty
	out	UDR,RS232_OUT_BUFFER ; send the byte
ret

RS232_OUT_STRING:
	in TEMP_REGS,SREG	;store SREG
	lpm
	tst	r0
	breq	ps_ret			; check for end of string (0x00)
	mov	r20,r0
	rcall	RS232_OUT		; send character
	adiw	ZL,1			; next char
	rjmp	RS232_OUT_STRING
ps_ret:
	ldi RS232_OUT_BUFFER, 13
	rcall RS232_OUT
	out SREG,TEMP_REGS	;recall SREG
ret


RS232_OUT_EEPROM_STRING:
	in TEMP_REGS,SREG	;store SREG
        out EEAR, TEMP                		; EEPROM-Adresse
        sbi EECR, EERE                    	; Lesevorgang starten
        in RS232_OUT_BUFFER, EEDR           ; gelesenes Byte nach "data"
        tst RS232_OUT_BUFFER                ; auf 0 (Stringende testen)
        breq RS232_OUT_EEPROM_STRING_END	; falls 0, Funktion beenden
        rcall RS232_OUT                    	; ansonsten Byte senden...
        inc TEMP	                       	; ... Adresse um 1 erhhen...
        rjmp RS232_OUT_EEPROM_STRING        ; ... und zum Anfang der Funktion
RS232_OUT_EEPROM_STRING_END:
		ldi RS232_OUT_BUFFER, 13
		rcall RS232_OUT
	out SREG,TEMP_REGS	;recall SREG
ret


ADDRESS_write:
	sbic EECR,EEWE		; Wait for completion of previous write
	rjmp ADDRESS_write
	ldi TEMP, 0x01		; Set up EEPROM-address in EEPROM-address register
	out EEARL, TEMP
	out EEDR,ADDRESS	; Write data to data register
	sbi EECR,EEMWE		; Write logical one to EEMWE
	sbi EECR,EEWE		; Start eeprom write by setting EEWE
ret


ADDRESS_read:
	sbic EECR,EEWE		; Wait for completion of previous write
	rjmp ADDRESS_read
	ldi TEMP, 0x01		; Set up EEPROM-address in EEPROM-address register
	out EEARL, TEMP
	sbi EECR,EERE		; Start eeprom read by writing EERE
	in ADDRESS,EEDR		; Read data from data register
ret

TEND:	ret
